@@ -2,38 +2,33 @@
 # 2.0, and the BSD License. See the LICENSE file in the root of this repository
 # for complete details.
 
-from __future__ import absolute_import, division, print_function
 
 import abc
 
-import six
 
-
-@six.add_metaclass(abc.ABCMeta)
-class AsymmetricSignatureContext(object):
+class AsymmetricSignatureContext(metaclass=abc.ABCMeta):
     @abc.abstractmethod
-    def update(self, data):
+    def update(self, data: bytes) -> None:
         """
         Processes the provided bytes and returns nothing.
         """
 
     @abc.abstractmethod
-    def finalize(self):
+    def finalize(self) -> bytes:
         """
         Returns the signature as bytes.
         """
 
 
-@six.add_metaclass(abc.ABCMeta)
-class AsymmetricVerificationContext(object):
+class AsymmetricVerificationContext(metaclass=abc.ABCMeta):
     @abc.abstractmethod
-    def update(self, data):
+    def update(self, data: bytes) -> None:
         """
         Processes the provided bytes and returns nothing.
         """
 
     @abc.abstractmethod
-    def verify(self):
+    def verify(self) -> None:
         """
         Raises an exception if the bytes provided to update do not match the
         signature or the signature does not match the public key.
